#include <cassert>
#include <iostream>
#include <iomanip>
#include <sstream>
#include <thread>

using namespace std;

// y = y + alpha * x
void axpy(int n, double alpha, const double* x, int incX, double* y, int incY) {
   for (int i = 0; i < n; ++i, x += incX, y += incY) {
      *y += alpha * *x;
   }
}

class AxpyThread {
   public:
      AxpyThread(int _n, double _alpha, double* _x, int _incX,
	 double* _y, int _incY) :
	    n(_n), alpha(_alpha), x(_x), incX(_incX),
	    y(_y), incY(_incY) {
      }
      void operator()() {
	 axpy(n, alpha, x, incX, y, incY);
      }
   private:
      int n;
      double alpha;
      double* x; int incX;
      double* y; int incY;
};

void mt_axpy(int n, double alpha, double* x, int incX, double* y,
             int incY, int nofthreads) {
   assert(n > 0 && nofthreads > 0);
   thread axpy_thread[nofthreads];

   // spawn threads and pass parameters
   int chunk = n / nofthreads;
   int remainder = n % nofthreads;
   int nextX = 0; int nextY = 0;
   for (int i = 0; i < nofthreads; ++i) {
      int len = chunk;
      if (i < remainder) ++len;
      axpy_thread[i] = thread(AxpyThread(len, alpha,
	 x + nextX * incX, incX, y + nextY * incY, incY));
      nextX += len; nextY += len;
   }

   // wait for all threads to finish
   for (int i = 0; i < nofthreads; ++i) {
      axpy_thread[i].join();
   }
}

char* cmdname;
void usage() {
   cerr << "Usage: " << cmdname << " [len [# threads]]" << endl;
   exit(1);
}

void fill_vector(double* a, int len) {
   static double val = 0;
   while (len > 0) {
      *a++ = val; val = val + 1;
      --len;
   }
}

int main(int argc, char** argv) {
   const double a = 0;
   const double b = 1;
   int N = 10000;
   int nofthreads = 10;

   cmdname = *argv++; --argc;
   if (argc > 0) {
      istringstream arg(*argv++); --argc;
      if (!(arg >> N) || N <= 0) usage();
   }
   if (argc > 0) {
      istringstream arg(*argv++); --argc;
      if (!(arg >> nofthreads) || nofthreads <= 0) usage();
   }
   if (argc > 0) usage();

   double* x = new double[N]; fill_vector(x, N);
   double* y = new double[N]; fill_vector(y, N);
   double alpha = 2;
   // axpy(N, alpha, x, 1, y, 1);
   mt_axpy(N, alpha, x, 1, y, 1, nofthreads);

#ifdef OUTPUT
   for (int i = 0; i < N; ++i) {
      if (i % 8 == 0) {
	 cout << endl;
      } else {
	 cout << " ";
      }
      cout << y[i];
   }
   cout << endl;
#endif
}
